extends Node

var player_name = ""
var server_ip = "127.0.0.1"
var server_port = "55555"
var players = [1]
var player_names = ["1"]

var net:NetworkedMultiplayerENet = null
var net_max_players:int = 6
var net_max_hp:int = 100
var net_pistol_reload_time = 100
var net_rocket_reload_time = 10
var net_pistol_damage = 5
var net_rocket_damage = 20
var net_mode = "host"

var wait_time = 120
var game_started = false

var ball_radius = 20
var ball_min_layers = 2
var ball_gravity = 10
var gen_seed = 0
var ball_layers = 15
var genscale = 5
var window_scale = 3
var player_speed = 2
var player_gravity = 10
var jump_force = 150
var player_floor_count = 0
var bullet_speed = 1000

func _ready():
	self.rpc_config("_sync_data",1)
	self.rpc_config("_update",1)
	self.rpc_config("_done",1)
	self.rpc_config("_data_back",1)

func _hosting():
	net = NetworkedMultiplayerENet.new()
	net.create_server(server_port, 6)
	get_tree().set_network_peer(net)
	net.connect("peer_connected", self, "_new_client")
	net.connect("peer_disconnected",self, "_delete_player_list")


func _connect():
	net = NetworkedMultiplayerENet.new()
	net.create_client(server_ip,int(server_port))
	get_tree().set_network_peer(net)
	net.connect("peer_connected", self, "_new_client")
	net.connect("peer_disconnected",self, "_delete_player_list")
	net.connect("server_disconnected", self, "_server_dis")


func _new_client(id):
	if (net_mode == "host"):
		if (len(players) == net_max_players or game_started):
			net.disconnect_peer(id, true)
		else:
			_update_player_list(id)

func _delete_player_list(id):
	if (players.has(id)):
		player_names.remove(players.find(id))
		players.remove(players.find(id))

func _update_player_list(id):
	rpc_id(id, "_sync_data", net.get_unique_id(), gen_seed, players, net_max_players, net_max_hp, net_pistol_reload_time, net_rocket_reload_time, net_pistol_damage, net_rocket_damage)

puppet func _data_back(id, name):
	player_names.append(name)
	players.append(id)
	rpc_id(id, "_done", wait_time, player_names)
	_update_data()

func _update_data():
	for client in players:
		rpc_id(client, "_update", players, player_names)

remotesync func _update(new_players, new_player_names):
	if (net_mode == "peer"):
		players = new_players
		player_names = new_player_names


puppet func _sync_data( id, gen_seed, players, net_max_players, net_max_hp, net_pistol_reload_time, net_rocket_reload_time, net_pistol_damage, net_rocket_damage):
	core.gen_seed = gen_seed
	core.net_max_players = net_max_players
	core.net_max_hp = net_max_hp
	core.net_pistol_reload_time = net_pistol_reload_time
	core.net_pistol_damage = net_pistol_damage
	core.net_rocket_reload_time = net_rocket_reload_time
	core.net_rocket_damage = net_rocket_damage
	
	rpc_id(id, "_data_back", core.net.get_unique_id(), core.player_name)

puppet func _done(new_wait_time, new_player_names):
	core.wait_time = new_wait_time
	core.player_names = new_player_names
	get_tree().change_scene("res://scenes/lobby_player.tscn")
	

func _server_dis():
	get_tree().set_network_peer(null)
	get_tree().change_scene("res://scenes/menu.tscn")

func _input(event):
	if (Input.is_action_just_pressed("exit")):
		_server_dis()
