extends RigidBody2D

var bullet_preload = preload("res://preload/bullet.tscn")
var rocket_preload = preload("res://preload/rocket.tscn")
var id = "player"

var pistol_ready = true
var rocket_ready = false
var shot = false

func _ready():
	#if (is_network_master() or len(core.players) == 1):
	self.gravity_scale = core.player_gravity
	$pistol.wait_time = core.net_pistol_reload_time * 0.001
	$rocket.wait_time = core.net_rocket_reload_time
	
	
	$stat/pistol.text = "Pistol: Ready"
	$stat/rocket.text = "Rocket: Reloading..."
	$rocket.start()
	
	self.rpc_config("_set_pos", 1)
	#self.rpc_config("_update_health",1)


func _gun_side(rot):
	if (90<rot and rot<270):
		$gun_rotation/ColorRect.hide()
		$gun_rotation/ColorRect2.hide()
		$gun_rotation/ColorRect3.show()
		$gun_rotation/ColorRect4.show()
	else:
		$gun_rotation/ColorRect.show()
		$gun_rotation/ColorRect2.show()
		$gun_rotation/ColorRect3.hide()
		$gun_rotation/ColorRect4.hide()

remote func _set_pos(pos, rot):
	global_transform.origin = pos
	$gun_rotation.rotation = deg2rad(rot)
	_gun_side(rot)


remote func _bullet(pos, rot, vel, body):
	var bullet  = bullet_preload.duplicate().instance()
	bullet.position = pos
	bullet.rotation = rot
	bullet.linear_velocity = vel
	bullet.creator = body
	get_parent().get_parent().add_child(bullet)

remote func _rocket(pos, rot, vel, body):
	var bullet  = rocket_preload.duplicate().instance()
	bullet.position = pos
	bullet.rotation = rot
	bullet.linear_velocity = vel
	get_parent().get_parent().add_child(bullet)


func _physics_process(delta):
	if (is_network_master()):
		var vel_y = self.linear_velocity.y
		var move = false
		
		if (Input.is_action_pressed("move_right")):
			linear_velocity = Vector2(150 * core.player_speed, vel_y)
			move = true
		elif (Input.is_action_pressed("move_left")):
			linear_velocity = Vector2(-150 * core.player_speed, vel_y)
			move = true
		if (Input.is_action_just_pressed("jump") and core.player_floor_count != 0):
			apply_impulse(Vector2(), Vector2(0, -5 * core.jump_force))
			move = true
		
		if (Input.is_action_just_pressed("shot") and pistol_ready):
			var bullet  = bullet_preload.duplicate().instance()
			var rotation_bullet = $gun_rotation.rotation - floor($gun_rotation.rotation / (2 * PI)) * 2 * PI
			if (move):
				rotation_bullet += rand_range(-0.1, 0.1)
			if (core.player_floor_count == 0):
				rotation_bullet += rand_range(-0.2, 0.2)
			bullet.position = $gun_rotation/Position2D.global_position
			bullet.rotation = rotation_bullet
			bullet.linear_velocity = Vector2(cos(rotation_bullet)*core.bullet_speed, sin(rotation_bullet)*core.bullet_speed)
			apply_impulse(Vector2(), Vector2(-cos(rotation_bullet)*75, -sin(rotation_bullet)*50))
			bullet.creator = name
			get_parent().get_parent().add_child(bullet)
			for player in core.players:
				if (player != core.net.get_unique_id()):
					rpc_id(player, "_bullet", $gun_rotation/Position2D.global_position, rotation_bullet, Vector2(cos(rotation_bullet)*core.bullet_speed, sin(rotation_bullet)*core.bullet_speed), name)
			pistol_ready = false
			$stat/pistol.text = "Pistol: Reloading..."
			$pistol.start()
		
		if (Input.is_action_just_pressed("shot_2") and rocket_ready):
			var bullet  = rocket_preload.duplicate().instance()
			var rotation_bullet = $gun_rotation.rotation - floor($gun_rotation.rotation / (2 * PI)) * 2 * PI
			if (move):
				rotation_bullet += rand_range(-0.1, 0.1)
			if (core.player_floor_count == 0):
				rotation_bullet += rand_range(-0.2, 0.2)
			bullet.position = $gun_rotation/Position2D.global_position
			bullet.rotation = rotation_bullet
			bullet.linear_velocity = Vector2(cos(rotation_bullet)*core.bullet_speed, sin(rotation_bullet)*core.bullet_speed)
			apply_impulse(Vector2(), Vector2(-cos(rotation_bullet)*250, -sin(rotation_bullet)*200))
			bullet.set_network_master(core.net.get_unique_id())
			get_parent().get_parent().add_child(bullet)
			for player in core.players:
				if (player != core.net.get_unique_id()):
					rpc_id(player, "_rocket", $gun_rotation/Position2D.global_position, rotation_bullet, Vector2(cos(rotation_bullet)*core.bullet_speed, sin(rotation_bullet)*core.bullet_speed), name)
			rocket_ready = false
			$stat/rocket.text = "Rocket: Reloading..."
			$rocket.start()
		
		$gun_rotation.look_at(get_global_mouse_position())
		_gun_side($gun_rotation.rotation_degrees - floor($gun_rotation.rotation_degrees / 360)*360)
		
		rpc_unreliable("_set_pos", global_transform.origin, $gun_rotation.rotation_degrees - floor($gun_rotation.rotation_degrees / 360)*360)


func on_floor(body):
	if (is_network_master() or len(core.players) == 1):
		core.player_floor_count += 1


func on_not_floor(body):
	if (is_network_master() or len(core.players) == 1):
		core.player_floor_count -= 1


func _reload_pistol():
	pistol_ready = true
	$stat/pistol.text = "Pistol: Ready"


func _reload_rocket():
	rocket_ready = true
	$stat/rocket.text = "Rocket: Ready"


func _shot(value):
	if (value <= 0):
		core._server_dis()
	if (!shot):
		for player in core.players:
			rpc_id(player, "_update_health", name, value, str(core.net.get_unique_id()))
	else:
		shot = false


remotesync func _update_health(id, health, sender):
	if (str(core.net.get_unique_id()) != sender):
		shot = true
		if (str(id) == str(name)):
			$hp.value = health
